@@ -3,6 +3,32 @@ import io
 import psycopg2
 from . import generic
 
+
+class InformationSchemaMapping (generic.SQLObjectMapping):
+    def __init__(self, db, query):
+        self.query = query
+        self.schema = (db.open_table("information_schema.tables")
+                       .view("select * from _ where table_schema not in ('information_schema', 'pg_catalog')")
+                       .view(query))
+        self.db = db
+        
+    def __contains__(self, key):
+        return bool(self.schema.table("select table_name from _ where table_name = %s", parameters=[key]))
+    
+    def __len__(self):
+        [[count]] = self.schema.view("select count(*) from _")
+        return count
+    
+    def __iter__(self):
+        return (r.table_name for r in self.schema.view("select * from _ order by table_name"))
+
+    def __getitem__(self, key):
+        if key in self:
+            return self.db.open_table(key)
+        else:
+            raise KeyError
+
+    
 class Database (generic.Database):
     """Connection to a PostgreSQL database.
 
@@ -17,6 +43,8 @@ class Database (generic.Database):
         self.name = name
         self.value_placeholder = "%s"
         self._in_transaction = False
+        self.tables = InformationSchemaMapping(self, "select * from _ where table_type = 'BASE TABLE'")
+        self.views = InformationSchemaMapping(self, "select * from _ where table_type = 'VIEW'")
 
     @contextmanager
     def _transaction(self):
@@ -44,7 +72,7 @@ class Database (generic.Database):
         if cursor_type == "server":
             cur.fetchmany(0) # Force PostgreSQL to populate cur.description
         return cur
-
+    
     def _insert_values(self, table, values, column_names=None):
         def quote_copy_text(value):
             if value is None:
