@@ -6,6 +6,29 @@ from . import generic
 logger = logging.getLogger(__name__)
 
 
+class SQLiteSchemaMapping (generic.SQLObjectMapping):
+    def __init__(self, db, query):
+        self.query = query
+        self.schema = db.open_table("sqlite_master").view(query)
+        self.db = db
+        
+    def __contains__(self, key):
+        return bool(self.schema.table("select name from _ where name = ?", parameters=[key]))
+    
+    def __len__(self):
+        [[count]] = self.schema.view("select count(*) from _")
+        return count
+    
+    def __iter__(self):
+        return (r.name for r in self.schema.view("select * from _ order by name"))
+
+    def __getitem__(self, key):
+        if key in self:
+            return self.db.open_table(key)
+        else:
+            raise KeyError
+
+
 class Database (generic.Database):
     """Connection to a SQLite database.
 
@@ -19,14 +42,16 @@ class Database (generic.Database):
         self.name = name
         self.default_column_type=""
         self.value_placeholder = "?"
-        self.temporary_prefix = "temp.temp_"        
+        self.temporary_prefix = "temp.temp_"
+        self.tables = SQLiteSchemaMapping(self, "select * from _ where type = 'table'")
+        self.views = SQLiteSchemaMapping(self, "select * from _ where type = 'view'")
     
     def _drop(self, statement):
         logger.debug(f"[{self!r}] Scheduling drop {statement!r}")        
         self._gc_statements.append(statement)
         self._garbage_collect()        
         
-    def drop_table(self, table_name, if_exists=False):
+    def drop_table(self, table_name):
         """Drop a table from the database. To work around locking issues, the table is first 
         renamed and then dropped once all active iterators on the connection have ended.
         
@@ -35,14 +60,8 @@ class Database (generic.Database):
         """        
         quoted_table_name = self.quote_name(table_name)
         temp_name = self._generate_temp_name(prefix="_drop_")
-        try:
-            with self._transaction():
-                self.execute(f"alter table {quoted_table_name} rename to {temp_name}")
-        except sqlite3.OperationalError as err:
-            if if_exists and len(err.args) > 0 and err.args[0].startswith("no such table:"):
-                return
-            else:
-                raise
+        with self._transaction():
+            self.execute(f"alter table {quoted_table_name} rename to {temp_name}")
         self._drop(f"drop table {temp_name}")        
         
     def create_function(self, name, nargs, fn):
